@@ -1,4 +1,4 @@
-# pylint: skip-file
+"""Jupyter notebook integration of Vizzu."""
 
 from .chart import Chart
 from .animation import (
@@ -11,6 +11,7 @@ from .animation import (
     Snapshot,
     AnimationMerger,
 )
-from .method import Method, Animate, Feature, Store
+from .method import Method, Animate, Feature, Store, EventOn, EventOff, Log
 from .json import RawJavaScript, RawJavaScriptEncoder
-from .template import DisplayTarget, DisplayTemplate
+from .template import ChartProperty, DisplayTarget, DisplayTemplate
+from .event import EventHandler
