@@ -1,31 +1,29 @@
-"""
-Jupyter notebook integration for Vizzu.
-"""
+"""A module for working with Vizzu charts."""
 
 import pkgutil
 import uuid
+from typing import List, Optional, Union
 
 from IPython.display import display_javascript
 from IPython import get_ipython
 
 from ipyvizzu.animation import Animation, Snapshot, AnimationMerger
-from ipyvizzu.method import Animate, Feature, Store
-from ipyvizzu.template import DisplayTarget, DisplayTemplate
+from ipyvizzu.method import Animate, Feature, Store, EventOn, EventOff, Log
+from ipyvizzu.template import ChartProperty, DisplayTarget, DisplayTemplate
+from ipyvizzu.event import EventHandler
 
 
 class Chart:
-    """
-    Wrapper over Vizzu Chart
-    """
+    """A class for representing a wrapper over Vizzu chart."""
 
     VIZZU = "https://cdn.jsdelivr.net/npm/vizzu@~0.4.0/dist/vizzu.min.js"
 
     def __init__(
         self,
-        vizzu=VIZZU,
-        width="800px",
-        height="480px",
-        display: DisplayTarget = DisplayTarget("actual"),
+        vizzu: Optional[str] = VIZZU,
+        width: Optional[str] = "800px",
+        height: Optional[str] = "480px",
+        display: Optional[DisplayTarget] = DisplayTarget.ACTUAL,
     ):
         self._chart_id = uuid.uuid4().hex[:7]
 
@@ -35,9 +33,6 @@ class Chart:
 
         self._scroll_into_view = False
 
-        if self._display_target != DisplayTarget.MANUAL:
-            self._register_events()
-
         ipyvizzujs = pkgutil.get_data(__name__, "templates/ipyvizzu.js").decode("utf-8")
         self._display(DisplayTemplate.IPYVIZZUJS.format(ipyvizzujs=ipyvizzujs))
 
@@ -50,28 +45,32 @@ class Chart:
             )
         )
 
+        if self._display_target != DisplayTarget.MANUAL:
+            self._register_events()
+
     @staticmethod
-    def _register_events():
+    def _register_events() -> None:
         ipy = get_ipython()
         if ipy is not None:
             ipy.events.register("pre_run_cell", Chart._register_pre_run_cell)
 
     @staticmethod
-    def _register_pre_run_cell():
+    def _register_pre_run_cell() -> None:
         display_javascript(DisplayTemplate.CLEAR_INHIBITSCROLL, raw=True)
 
     @property
-    def scroll_into_view(self):
+    def scroll_into_view(self) -> bool:
+        """A property for turning on/off scroll into view."""
+
         return self._scroll_into_view
 
     @scroll_into_view.setter
-    def scroll_into_view(self, scroll_into_view):
+    def scroll_into_view(self, scroll_into_view: Optional[bool]):
         self._scroll_into_view = bool(scroll_into_view)
 
-    def animate(self, *animations: Animation, **options):
-        """
-        Show new animation.
-        """
+    def animate(self, *animations: Animation, **options: Optional[dict]) -> None:
+        """A method for animating the chart."""
+
         if not animations:
             raise ValueError("No animation was set.")
 
@@ -80,7 +79,7 @@ class Chart:
 
         self._display(
             DisplayTemplate.ANIMATE.format(
-                display_target=self._display_target,
+                display_target=self._display_target.value,
                 chart_id=self._chart_id,
                 scroll=str(self._scroll_into_view).lower(),
                 **animate.dump(),
@@ -88,7 +87,9 @@ class Chart:
         )
 
     @staticmethod
-    def _merge_animations(animations):
+    def _merge_animations(
+        animations: List[Animation],
+    ) -> Union[Animation, AnimationMerger]:
         if len(animations) == 1:
             return animations[0]
 
@@ -98,7 +99,9 @@ class Chart:
 
         return merger
 
-    def feature(self, name, enabled):
+    def feature(self, name: str, enabled: bool) -> None:
+        """A method for turning on/off a feature of the chart."""
+
         self._display(
             DisplayTemplate.FEATURE.format(
                 chart_id=self._chart_id,
@@ -107,6 +110,8 @@ class Chart:
         )
 
     def store(self) -> Snapshot:
+        """A method for saving and storing the actual state of the chart."""
+
         snapshot_id = uuid.uuid4().hex[:7]
         self._display(
             DisplayTemplate.STORE.format(
@@ -115,10 +120,43 @@ class Chart:
         )
         return Snapshot(snapshot_id)
 
-    def _repr_html_(self):
+    def on(  # pylint: disable=invalid-name
+        self, event: str, handler: str
+    ) -> EventHandler:
+        """A method for creating and turning on an event handler."""
+
+        event_handler = EventHandler(event, handler)
+        self._display(
+            DisplayTemplate.SET_EVENT.format(
+                chart_id=self._chart_id,
+                **EventOn(event_handler).dump(),
+            )
+        )
+        return event_handler
+
+    def off(self, event_handler: EventHandler) -> None:
+        """A method for turning off an event handler."""
+
+        self._display(
+            DisplayTemplate.CLEAR_EVENT.format(
+                chart_id=self._chart_id,
+                **EventOff(event_handler).dump(),
+            )
+        )
+
+    def log(self, chart_property: ChartProperty) -> None:
+        """A method for printing chart properties to the browser console."""
+
+        self._display(
+            DisplayTemplate.LOG.format(
+                chart_id=self._chart_id, **Log(chart_property).dump()
+            )
+        )
+
+    def _repr_html_(self) -> str:
         assert (
             self._display_target == DisplayTarget.MANUAL
-        ), f'chart._repr_html_() can be used with display="{DisplayTarget.MANUAL}" only'
+        ), f'chart._repr_html_() can be used with display="{DisplayTarget.MANUAL.value}" only'
         assert not self._showed, "cannot be used after chart displayed."
         self._showed = True
         html_id = uuid.uuid4().hex[:7]
@@ -131,10 +169,12 @@ class Chart:
         )
         return f'<div id="{html_id}"><script>{script}</script></div>'
 
-    def show(self):
+    def show(self) -> None:
+        """A method for displaying the assembled javascript code."""
+
         assert (
             self._display_target == DisplayTarget.MANUAL
-        ), f'chart.show() can be used with display="{DisplayTarget.MANUAL}" only'
+        ), f'chart.show() can be used with display="{DisplayTarget.MANUAL.value}" only'
         assert not self._showed, "cannot be used after chart displayed"
         display_javascript(
             "\n".join(self._calls),
@@ -142,12 +182,12 @@ class Chart:
         )
         self._showed = True
 
-    def _display(self, javascript):
+    def _display(self, javascript: str) -> None:
         if self._display_target != DisplayTarget.MANUAL:
             display_javascript(
                 javascript,
                 raw=True,
             )
         else:
-            assert not self._showed, "cannot be used after chart.show()"
+            assert not self._showed, "cannot be used after chart displayed"
             self._calls.append(javascript)
