@@ -1,7 +1,7 @@
 if (!window.IpyVizzu) {
     class IpyVizzu 
     {
-        constructor(element, chartId, vizzulib, divWidth, divHeight)
+        constructor()
         {
             IpyVizzu.inhibitScroll = false;
             IpyVizzu.nbconvert = false;
@@ -10,15 +10,12 @@ if (!window.IpyVizzu) {
             document.addEventListener('touchstart', (evt) => { IpyVizzu.inhibitScroll = true }, true);
 
             this.elements = {};
-            this.elements[chartId] = document.createElement("div");
-            this.elements[chartId].style.cssText = `width: ${divWidth}; height: ${divHeight};`;
-
             this.charts = {};
-            this.charts[chartId] = import(vizzulib).then(Vizzu => new Vizzu.default(this.elements[chartId]).initializing);
-            this._moveHere(chartId, element);
-
+            
             this.snapshots = {};
             this.displays = {};
+
+            this.events = {};
         }
 
         static clearInhibitScroll(element)
@@ -27,6 +24,13 @@ if (!window.IpyVizzu) {
             IpyVizzu.inhibitScroll = false;
         }
 
+        createChart(element, chartId, vizzulib, divWidth, divHeight) {
+            this.elements[chartId] = document.createElement("div");
+            this.elements[chartId].style.cssText = `width: ${divWidth}; height: ${divHeight};`;
+            this.charts[chartId] = import(vizzulib).then(Vizzu => new Vizzu.default(this.elements[chartId]).initializing);
+            this._moveHere(chartId, element);
+        }
+
         animate(element, chartId, displayTarget, scrollEnabled, chartTarget, chartAnimOpts)
         {
             if (IpyVizzu.nbconvert) IpyVizzu._hide(element);
@@ -34,6 +38,7 @@ if (!window.IpyVizzu) {
             this.charts[chartId] = this.charts[chartId].then(chart => {
                 if (displayTarget === 'actual') this._moveHere(chartId, element);
                 this._scroll(chartId, scrollEnabled);
+                if (typeof chartTarget === 'string') chartTarget = this.snapshots[chartTarget];
                 return chart.animate(chartTarget, chartAnimOpts);
             });
         }
@@ -47,12 +52,6 @@ if (!window.IpyVizzu) {
             });
         }
 
-        stored(element, id)
-        {
-            if (IpyVizzu.nbconvert) IpyVizzu._hide(element);
-            return this.snapshots[id];
-        }
-
         feature(element, chartId, name, enabled)
         {
             if (IpyVizzu.nbconvert) IpyVizzu._hide(element);
@@ -62,6 +61,34 @@ if (!window.IpyVizzu) {
             });
         }
 
+        setEvent(element, chartId, id, event, handler)
+        {
+            if (IpyVizzu.nbconvert) IpyVizzu._hide(element);
+            this.charts[chartId] = this.charts[chartId].then(chart => {
+                this.events[id] = handler;
+                chart.on(event, this.events[id]);
+                return chart;
+            });
+        }
+
+        clearEvent(element, chartId, id, event)
+        {
+            if (IpyVizzu.nbconvert) IpyVizzu._hide(element);
+            this.charts[chartId] = this.charts[chartId].then(chart => {
+                chart.off(event, this.events[id]);
+                return chart;
+            });
+        }
+
+        log(element, chartId, chartProperty)
+        {
+            if (IpyVizzu.nbconvert) IpyVizzu._hide(element);
+            this.charts[chartId] = this.charts[chartId].then(chart => {
+                console.log(chart[chartProperty])
+                return chart;
+            });
+        }
+
         _moveHere(chartId, element)
         {
             if (IpyVizzu.nbconvert) IpyVizzu._display(this.elements[chartId], element);
@@ -89,4 +116,5 @@ if (!window.IpyVizzu) {
     }
 
     window.IpyVizzu = IpyVizzu;
+    window.ipyvizzu = new window.IpyVizzu();
 }
\ No newline at end of file
